/***************************************************************
From the book "Motif Programming, The Essentials...and More"
by Marshall Brain.
Published by Digital Press, ISBN 1-55558-089-0.
To order the book, call 1-800-DIGITAL and ask for EY-J816E-DP.

Copyright 1992, by Digital Equipment Corp.

This code demonstrates the use of an input callback on an
incoming stream of data. Compile "sample.c" before running
this program.
***************************************************************/

/* input.c */

#include <Xm/Xm.h>
#include <Xm/SelectioB.h>

#define    OK     1
#define CANCEL    2

XtAppContext context;
XmStringCharSet char_set=XmSTRING_DEFAULT_CHARSET;

Widget toplevel, dialog, list;
int pipefd1[2];
int pid;

void dialogCB(w,client_data,call_data)
    Widget w;
    int client_data;
    XmSelectionBoxCallbackStruct *call_data;
/* callback function for the selection box */
{
    char *s;

    switch (client_data)
    {
        case OK:
            /* get the string selected by the user */
            XmStringGetLtoR(call_data->value,char_set,&s);
            printf("%s\n",s);
            XtFree(s);
            break;
        case CANCEL:
            exit(0);
            break;
    }
}

void handle_input(client_data, source, id)
    XtPointer client_data;
    int *source;
    XtInputId *id;
{
    char *t,s[10000];
    XmString xs;
    int len;

    len=read(*source,s,10000);
    /* If the len==0, then the child has terminated so kill
       off the input event. Otherwise, parse the block into its
       separate lines and place them into the list widget. The
       parsing is necessary because the block will probably
       contain several lines rather than just one. */
    if (len==0) 
    {
        XtRemoveInput(*id);
        printf("Exec-ed program done.\n");
    }
    else
    {
        s[len]='\0';
        t=strtok(s,"\n");
        while (t)
        {
            xs=XmStringCreateLtoR(t,char_set);
            XmListAddItem(list,xs);
            XmStringFree(xs);
            printf("%s\n",t);
            t=strtok(NULL,"\n");
        }
    }
}

void setup_input_event()
{
    Arg al[10];
    int ac;

    pipe(pipefd1);
    if((pid=fork())==0)/*child*/
    {
        close(pipefd1[0]);
        close(1);
        dup(pipefd1[1]);
        execlp("sample","sample",(char*)0);
    }
    else
    {
        close(pipefd1[1]); 
        XtAppAddInput(context,pipefd1[0],XtInputReadMask,
            handle_input,NULL);
    }
}

void main(argc,argv)
    int argc; 
    char *argv[];
{
    Arg al[10];
    int ac;

    /* create the toplevel shell */
    toplevel = XtAppInitialize(&context,"",NULL,0,&argc,argv,
        NULL,NULL,0);

    /* create the selection box dialog */
    ac = 0;
    XtSetArg(al[ac],XmNautoUnmanage,False); ac++;
    XtSetArg(al[ac],XmNmustMatch,True); ac++;
    XtSetArg(al[ac],XmNselectionLabelString, 
        XmStringCreateLtoR("Pick a process to kill.",char_set)); ac++;
    dialog=XmCreateSelectionBox(toplevel,"dialog",al,ac);
    XtManageChild(dialog);
    XtAddCallback(dialog,XmNokCallback, dialogCB,OK);
    XtAddCallback(dialog,XmNcancelCallback,dialogCB,CANCEL);
    XtUnmanageChild(XmSelectionBoxGetChild(dialog,
        XmDIALOG_HELP_BUTTON));
    XtUnmanageChild(XmSelectionBoxGetChild(dialog,
         XmDIALOG_APPLY_BUTTON));
    list=XmSelectionBoxGetChild(dialog,XmDIALOG_LIST);

    setup_input_event();

    XtRealizeWidget(toplevel);
    XtAppMainLoop(context);
}
